@@ -25,7 +25,8 @@
 
 #include <stdio.h>
 
-#include <config.h>
+#include <gnumeric-config.h>
+#include <gnumeric.h>
 #include "boot.h"
 #include "ms-escher.h"
 #include "escher-types.h"
