@@ -5,17 +5,19 @@
  *  Miguel de Icaza (miguel@gnu.org)
  *  Jukka-Pekka Iivonen (iivonen@iki.fi)
  */
-#include <config.h>
-#include "func.h"
-#include "func-util.h"
-#include "parse-util.h"
-#include "cell.h"
-#include "sheet.h"
-#include "str.h"
-#include "mathfunc.h"
-#include "rangefunc.h"
-#include "collect.h"
-#include "auto-format.h"
+#include <gnumeric-config.h>
+#include <gnumeric.h>
+#include <func.h>
+
+#include <func-util.h>
+#include <parse-util.h>
+#include <cell.h>
+#include <sheet.h>
+#include <str.h>
+#include <mathfunc.h>
+#include <rangefunc.h>
+#include <collect.h>
+#include <auto-format.h>
 
 #include <math.h>
 #include <string.h>
