@@ -13,18 +13,10 @@
  * Author:
  *   Miguel de Icaza (miguel@gnu.org)
  */
-#include <config.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <glib.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
-#include <string.h>
-#include <ctype.h>
-#include <errno.h>
-#include <locale.h>
-#include <math.h>
+#include <gnumeric-config.h>
+#include "gnumeric.h"
 #include "number-match.h"
+
 #include "formats.h"
 #include "dates.h"
 #include "numbers.h"
@@ -36,6 +28,14 @@
 #include "mathfunc.h"
 #include "str.h"
 
+#include <libgnome/gnome-i18n.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <ctype.h>
+#include <errno.h>
+#include <locale.h>
+#include <math.h>
 #undef DEBUG_NUMBER_MATCH
 
 /*
@@ -1122,7 +1122,7 @@ format_match_simple (char const *text)
  * Attempts to parse the supplied string to see if it matches a known value format.
  * If @format is supplied it will get a copy of the matching format with no
  * additional references.   The caller is responsible for releasing the
- * resulting value.
+ * resulting value.  @matching_format does NOT have a reference added.
  */
 Value *
 format_match (char const *text, StyleFormat *cur_fmt,
