@@ -6,8 +6,10 @@
  *   Miguel de Icaza (miguel@gnu.org).
  *   Michael Meeks   (mmeeks@gnu.org).
  */
-#include <config.h>
+#include <gnumeric-config.h>
+#include "gnumeric.h"
 #include "ranges.h"
+
 #include "numbers.h"
 #include "expr.h"
 #include "expr-name.h"
@@ -99,20 +101,18 @@ range_parse (Sheet *sheet, char const *range, gboolean strict)
  * Should be merged with range_parse
  */
 gboolean
-parse_range (char const *text, int *start_col, int *start_row,
-	     int *end_col, int *end_row)
+parse_range (char const *text, Range *r)
 {
 	int len;
 
-	if (!parse_cell_name (text, start_col, start_row, FALSE, &len))
+	if (!parse_cell_name (text, &r->start.col, &r->start.row, FALSE, &len))
 		return FALSE;
 	if (text [len] == '\0') {
-		*end_col = *start_col;
-		*end_row = *start_row;
+		r->end = r->start;
 		return TRUE;
 	}
 	return (text[len] == ':' &&
-		parse_cell_name (text + len + 1, end_col, end_row, TRUE, NULL));
+		parse_cell_name (text + len + 1, &r->end.col, &r->end.row, TRUE, NULL));
 }
 
 /**
@@ -250,7 +250,8 @@ range_name (Range const *src)
  * Return value: Whether or not the range has a header
  **/
 gboolean
-range_has_header (Sheet const *sheet, Range const *src, gboolean top)
+range_has_header (Sheet const *sheet, Range const *src,
+		  gboolean top, gboolean ignore_styles)
 {
 	Cell *ca, *cb;
 	Value *valuea, *valueb;
@@ -302,12 +303,14 @@ range_has_header (Sheet const *sheet, Range const *src, gboolean top)
 			}
 		}
 
-		/* Look for style differences */
-		stylea = cell_get_mstyle (ca);
-		styleb = cell_get_mstyle (cb);
+		if (!ignore_styles) {
+			/* Look for style differences */
+			stylea = cell_get_mstyle (ca);
+			styleb = cell_get_mstyle (cb);
 
-		if (!mstyle_equal (stylea, styleb))
-			return TRUE;
+			if (!mstyle_equal (stylea, styleb))
+				return TRUE;
+		}
 	}
 
 	return FALSE;
@@ -1001,23 +1004,30 @@ global_range_dup (GlobalRange const *src)
 Value *
 global_range_parse (Sheet *sheet, char const *str)
 {
-	Value    *value = NULL;
-	ExprTree *expr = NULL;
 	ParsePos  pp;
+	ExprTree *expr;
+
+	g_return_val_if_fail (IS_SHEET (sheet), NULL);
+	g_return_val_if_fail (str != NULL, NULL);
+
+	expr = expr_parse_str (str,
+		parse_pos_init (&pp, sheet->workbook, sheet, 0, 0),
+		GNM_PARSER_FORCE_EXPLICIT_SHEET_REFERENCES,
+		NULL, NULL);
 
-	parse_pos_init (&pp, sheet->workbook, sheet, 0, 0);
-	expr = expr_parse_string (str, &pp, NULL, NULL);
 	if (expr != NULL)  {
-		value = expr_tree_get_range (expr);
+		Value * value = expr_tree_get_range (expr);
 		expr_tree_unref (expr);
+		return value;
 	}
-	return value;
+
+	return NULL;
 }
 
 /**
  * global_range_list_parse:
  * @sheet: Sheet where the range specification is relatively parsed to
- * @range_spec: a range or list of ranges to parse (ex: "A1", "A1:B1,C2,Sheet2!D2:D4")
+ * @str  : a range or list of ranges to parse (ex: "A1", "A1:B1,C2,Sheet2!D2:D4")
  *
  * Parses a list of ranges, relative to the @sheet and returns a list with the
  * results.
@@ -1025,33 +1035,41 @@ global_range_parse (Sheet *sheet, char const *str)
  * Returns a GSList containing Values of type VALUE_CELLRANGE, or NULL on failure
  */
 GSList *
-global_range_list_parse (Sheet *sheet, char const *range_spec)
+global_range_list_parse (Sheet *sheet, char const *str)
 {
-
-	char *copy, *range_copy, *r;
-	GSList *ranges = NULL;
+	ParsePos  pp;
+	ExprTree *expr;
+	GSList   *ranges = NULL;
+	Value	 *v;
 
 	g_return_val_if_fail (IS_SHEET (sheet), NULL);
-	g_return_val_if_fail (range_spec != NULL, NULL);
+	g_return_val_if_fail (str != NULL, NULL);
 
-	range_copy = copy = g_strdup (range_spec);
+	expr = expr_parse_str (str,
+		parse_pos_init (&pp, sheet->workbook, sheet, 0, 0),
+		GNM_PARSER_FORCE_EXPLICIT_SHEET_REFERENCES |
+		GNM_PARSER_PERMIT_MULTIPLE_EXPRESSIONS, NULL, NULL);
 
-	while ((r = strtok (range_copy, ",")) != NULL){
-		Value *v;
-
-		v = global_range_parse (sheet, r);
-		if (!v){
-			range_list_destroy (ranges);
-			g_free (copy);
-			return NULL;
+	if (expr != NULL)  {
+		if (expr->any.oper == OPER_SET) {
+			ExprList *l;
+			for (l = expr->set.set; l != NULL; l = l->next) {
+				v = expr_tree_get_range (l->data);
+				if (v == NULL) {
+					range_list_destroy (ranges);
+					ranges = NULL;
+					break;
+				} else
+					ranges = g_slist_prepend (ranges, v);
+			}
+		} else {
+			v = expr_tree_get_range (expr);
+			if (v != NULL)
+				ranges = g_slist_prepend (ranges, v);
 		}
-
-		ranges = g_slist_prepend (ranges, v);
-		range_copy = NULL;
+		expr_tree_unref (expr);
 	}
 
-	g_free (copy);
-
 	return g_slist_reverse (ranges);
 }
 
