@@ -8,12 +8,12 @@
  *    Jody Goldberg (jody@gnome.org)
  *
  * (C) 1998, 1999, 2000 Miguel de Icaza
- * (C) 2000-2001 Helix Code, Inc.
+ * (C) 2000-2001 Ximian, Inc.
  */
-#include <config.h>
-
-#include <stdlib.h>
+#include <gnumeric-config.h>
+#include "gnumeric.h"
 #include "workbook.h"
+
 #include "workbook-view.h"
 #include "workbook-control.h"
 #include "workbook-private.h"
@@ -39,10 +39,10 @@
 #include "sheet-object-bonobo.h"
 #endif
 
-#include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
 #include <gal/util/e-util.h>
 #include <ctype.h>
+#include <stdlib.h>
 
 static GtkObjectClass *workbook_parent_class;
 
@@ -142,7 +142,6 @@ workbook_destroy (GtkObject *wb_object)
 	wb->redo_commands = NULL;
 
 	workbook_deps_destroy (wb);
-	expr_name_invalidate_refs_wb (wb);
 
 	/* Copy the set of sheets, the list changes under us. */
 	sheets = workbook_sheets (wb);
@@ -228,6 +227,9 @@ workbook_set_dirty (Workbook *wb, gboolean is_dirty)
 {
 	g_return_if_fail (wb != NULL);
 
+	wb->modified = is_dirty;
+	if (wb->summary_info != NULL)
+		wb->summary_info->modified = is_dirty;
 	g_hash_table_foreach (wb->sheet_hash_private,
 			      cb_sheet_mark_dirty, GINT_TO_POINTER (is_dirty));
 }
@@ -253,6 +255,8 @@ workbook_is_dirty (Workbook const *wb)
 	gboolean dirty = FALSE;
 
 	g_return_val_if_fail (wb != NULL, FALSE);
+	if (wb->summary_info != NULL && wb->summary_info->modified)
+		return TRUE;
 
 	g_hash_table_foreach (wb->sheet_hash_private, cb_sheet_check_dirty,
 			      &dirty);
@@ -370,6 +374,7 @@ workbook_init (GtkObject *object)
 	wb->names        = NULL;
 	wb->summary_info   = summary_info_new ();
 	summary_info_default (wb->summary_info);
+	wb->summary_info->modified = FALSE;
 
 	/* Nothing to undo or redo */
 	wb->undo_commands = wb->redo_commands = NULL;
@@ -1084,23 +1089,24 @@ workbook_sheet_delete (Sheet *sheet)
 
 	wb = sheet->workbook;
 
-	/*
-	 * FIXME : Deleting a sheet plays havoc with our data structures.
-	 * Be safe for now and empty the undo/redo queues
-	 */
-	command_list_release (wb->undo_commands);
-	command_list_release (wb->redo_commands);
-	wb->undo_commands = NULL;
-	wb->redo_commands = NULL;
-	WORKBOOK_FOREACH_CONTROL (wb, view, control,
-		wb_control_undo_redo_clear (control, TRUE);
-		wb_control_undo_redo_clear (control, FALSE);
-		wb_control_undo_redo_labels (control, NULL, NULL);
-	);
+	if (!sheet->pristine) {
+		/*
+		 * FIXME : Deleting a sheet plays havoc with our data structures.
+		 * Be safe for now and empty the undo/redo queues
+		 */
+		command_list_release (wb->undo_commands);
+		command_list_release (wb->redo_commands);
+		wb->undo_commands = NULL;
+		wb->redo_commands = NULL;
+		WORKBOOK_FOREACH_CONTROL (wb, view, control,
+			wb_control_undo_redo_clear (control, TRUE);
+			wb_control_undo_redo_clear (control, FALSE);
+			wb_control_undo_redo_labels (control, NULL, NULL);
+		);
+	}
 
 	/* Important to do these BEFORE detaching the sheet */
 	sheet_deps_destroy (sheet);
-	expr_name_invalidate_refs_sheet (sheet);
 
 	/* All is fine, remove the sheet */
 	workbook_sheet_detach (wb, sheet);
